// ************************************************************************************************
//
// Copyright www.jb-electronics.de
//
// Nixiefish light sculpture
//
// using the PIC16F627
//
// (x) cycles through all digits of a Nixie tube
//     in a slot machine like effect
// (x) dims the brightness up and down periodically
// 
// http://www.jb-electronics.de/html/elektronik/nixies/n_nixiefish.htm
//
// ************************************************************************************************


// ************************************************************************************************
// include PIC header
#include <pic.h>


// ************************************************************************************************
// global variables

// used to cycle through the digits
int i = 0, counter = 0;
unsigned char digit = 0;
char digit_speed = 1;

// use this array if you connect the inputs of the K155ID1 differently
// here: ABCD
unsigned char digit_map[10] = {0b0000, 0b0001, 0b0010, 0b0011, 0b0100, 0b0101, 0b0110, 0b0111, 0b1000, 0b1001};

// alternatively: CBDA
//unsigned char digit_map[10] = {0b0000, 0b0001, 0b0100, 0b0101, 0b1000, 0b1001, 0b1100, 0b1101, 0b0010, 0b0011};

// this is the digit stack of the ZM1120,
// digit 4 is foremost, then digit 9, and so on...
// this array is conveniently used for slot machine effects
unsigned char digit_stack[10] = {4, 9, 3, 8, 5, 2, 6, 0, 7, 1};

// used to cycle through the PWM duty cycles
char pwm_speed = 1;
unsigned char pwm_value = 0;
int pwm_counter = 0;


// ************************************************************************************************
// main method
void main (void) {


    // *****
	// configuration (internal clock enabled)
	__CONFIG(0x3F70);


	// *****
	// ports for the shift register
	TRISB0 = 0;
	TRISB1 = 0;
	TRISB2 = 0;
	#define	STROBE	RB0
	#define	DATA	RB1
	#define	CLK		RB2


	// *****
	// configure TIMER0 (no prescaler, internal source)
	T0CS = 0;
	PSA = 1;
	PS0 = 0;
	PS1 = 0;
	PS2 = 0;

	// generate interrupt on TIMER1 overflow
	T0IE = 1;

	// enable global interrupts
	GIE = 1;


	// *****
	// PWM settings

	// time base for TIMER2 is 255
	PR2 = 0xff; // 255

	// CCP1 (RB3) is the output
	TRISB3 = 0;

	// set TIMER2 to 1:1 setzen (00 = 1:1, 01 = 1:4, 1X = 1:16)
	// and enable TIMER2 (most significant bit)
	T2CON = 0b100;

	// set CCP1 module to PWM mode
	CCP1CON = 0b1100;



	// *****
	// main loop
	while (1) {


		// send the current display data
		for (i = 7; i >= 0; i--) {
			if ((digit_map[digit_stack[digit]] << 4) & (1 << i)) {
				DATA = 1;
			} else {
			 	DATA = 0;
			}
			NOP ();
			CLK = 1;
			NOP ();
			CLK = 0;
		}
		NOP ();
		STROBE = 1;
		NOP ();
		STROBE = 0;


		// update the PWM value
		CCPR1L = pwm_value;


	}


}


// ************************************************************************************************
// interrupt service routine
static void interrupt isr (void) {


	// *****
	// this is called ~3900 times per second
	if (T0IF) {

		// PWM animation
		pwm_counter++;
		if (pwm_counter > 100) {
			pwm_counter = 0;
			pwm_value += pwm_speed;
			if (pwm_value >= 255) {
				pwm_speed = -1;
			} else if (pwm_value == 0) {
				pwm_speed = 1;
			}
		}	

		// slot machine animation
		counter++;
		if (counter > 200) {
			counter = 0;
			digit += digit_speed;
			if (digit >= 10) {
				digit_speed = -1;
			} else if (digit == 0) {
				digit_speed = 1;
			}
		}

		// reset interrupt flag
		T0IF = 0;

    }


}